function b=downsample(a,nv)
//a: [hyper]array of any size
//nv: vector of downsampling factors 
// an exercise about flattened representations of hypermatrices

//argument check and straightening of odd cases
na=matrix(size(a),1,-1);   // vector of the dimensions of a
lna=length(na); //rank of a
lnv=length(nv); nv=matrix(nv,1,-1);

//case of a vector (or a hypermat with one dimension ==1) and nv scalar
if lnv==1  then
  nv=nv*ones(1,lna); 
  lnv=lna;  
  nv(na==1)=1
end

if lna<>lnv then
  write(%io(2),'sizes of a and nv do not match')
  write(%io(2),'  a is of rank '+string(lna)+' and nv has '+..
        string(lnv)+' elements')
  abort
end

if or(nv>na) then
  write(%io(2),"nv larger than na : '+string(nv(nv>na))+..
             ">"+string(na(nv>na)))
  abort
end

//discarding edges
na=max(int(na./nv).*nv,1);
s=":,"; s1=",:";
for i=1:lna
//using string tricks to reference an arbitrary dimensional range is
// certainly unefficient, but is coding easy.
  str=sprintf("a=a("+strcat(s(ones(1,i-1)))+"1:"+string(na(i))+..
                strcat(s1(ones(1,lna-i)))+")")
  execstr(str) 
end

nna=prod(na); // total number of elements of a

//unroll the hypermat and average nv(1) consecutive elements
a=mean(matrix(a,nv(1),nna/nv(1)),1)

//iteratively roll up the unrolled hypermat in such a way that the
// designed neighbors end up on the same row and average that 
// (averaging is associative!)
for i=2:lna
//rearrange a so that nv(i) neighbors in the i-th dimension become
// consecutive in the flattened form
  nxy=nna/prod(nv(1:i-1)); ny=nxy/prod(na(i:$)); nvy=ny*nv(i)
  j=0:(nxy-1);
  j= 1 + ny*modulo(j,nv(i)) + int(modulo(j,nvy)/nv(i)) + int(j/nvy)*nvy
  a=a(j)
//average nv(i) consecutive elements
  a=mean(matrix(a,nv(i),-1),1)
end

//cast again in hypermat
b=hypermat(na./nv,a);

//return to array type instead of hypermat, if needed
if lna<3 then b=matrix(b,size(b,1),size(b,2)); end
