function pixmapl(varargin)

// sintaxes: pixmapl([x,y],a,[mode])
//           pixmapl([x,y],a,l,[mode])
//           pixmapl([x,y],a,amin,amax,[mode])
//           pixmapl([x,y],a,amin,amax,l,[mode])
//           pixmapl([x,y],a,amin,amax,l,nliv,[mode])
//draws the pixmap of the matrix a, together with a colorscale bar
// the matrix is rescaled so that the whole colormap range is used
//  xgetech and xsetech are used to conserve the current drawing subwindow
// if amin>=amax auto rescaling
// if l mod 4 <=2 no colorbar is drawn
// if l=odd, nliv contour lines are drawn (nliv=10 default)
//  i.e.: l=-1 -> no colorbar but contour
//        l=0  -> no colorbar nor contour
//        l=1  -> colorbar and contour
//        l=2  -> colorbar and no contour
//  if l=4,5,6,7 the (undocumented) function contourf rather
//  than the plot of each pixel is used (the result may be
//   more economical in terms of graphic content, but on the
//   other side it's an interpolation of the original array)

//parsing of the input arguments:
[rhs,lhs]=argn(0);
if lhs==0 then
   demo=['   x=ones(100,1)*linspace(-6,6,100)';
         '   pixmapl(cos(x'').*exp(-x.^2/18),1,''sq'') ']
   write(%io(2),'Demo of pixmapl')
   write(%io(2),demo);
   execstr(demo)
   return
end

aspect='sq';
if lhs>1 then
   for i=1:lhs; 
     if type(varargin(i))==10 then 
        aspect=varargin(i);
        varargin(i)=null(); lhs=lhs-1; break
     end
   end
end
if lhs==1 then a=varargin(1); end
if lhs==2 then a=varargin(1); l=varargin(2); end
if lhs==3 then 
// note the trick prod(size()) instead of length(): length()
//  is not yet (2.5.1) overloaded for integers
  if prod([size(varargin(1)),size(varargin(2))])...
          ==prod(size(varargin(3)))  then
      x0=varargin(1); y0=varargin(2); a=varargin(3);
  else
      a=varargin(1); amin=varargin(2); amax=varargin(3);
  end
end

if lhs==4 then 
  if prod([size(varargin(1)),size(varargin(2))])...
          ==prod(size(varargin(3)))  then
      x0=varargin(1); y0=varargin(2); a=varargin(3);
  else
      a=varargin(1); amin=varargin(2); amax=varargin(3);
  end
  l=varargin(4);
end

if lhs==5 then 
  if prod([size(varargin(1)),size(varargin(2))])...
          ==prod(size(varargin(3)))  then
      x0=varargin(1); y0=varargin(2); a=varargin(3);
      amin=varargin(4); amax=varargin(5);
  else
      a=varargin(1); amin=varargin(2); amax=varargin(3);
      l=varargin(4); nliv=varargin(5);
  end
end

if lhs>=6 then
   x0=varargin(1); y0=varargin(2); a=varargin(3);
   amin=varargin(4); amax=varargin(5); l=varargin(6);
end
if lhs>=7 then nliv=varargin(7); end

// this is for working with 2d cuts of hypermatrices
idim=find(size(a)<>1); 
if length(idim)==0 then adim=[1,1]; end
if length(idim)==1 then 
  if idim==1 then adim=[size(a,idim),1]; end
  if idim>1 then adim=[1,size(a,idim)]; end
  a=matrix(a,adim(1),adim(2));
end;
if length(idim)==2 then
 adim=[size(a,idim(1)),size(a,idim(2))];
 a=matrix(a,adim(1),adim(2));
end
if length(idim)>2 then
  write(%io(2),' I can only plot 2d arrays, data is a '+...
       string(length(idim))+'d hypermatrix')
  return
end

// this is for getting through complex matrices
if type(a)==1 then 
  if ~isreal(a) then
    write(%io(2),' Data must be real, I''ll take the its real part')   
    a=real(a)
  end
end
//this for boolean a
if type(a)==4 then 
   a=bool2s(a); 
end

ax=size(a,1); ay=size(a,2);

if ~exists("nliv","local") then nliv=10; end
if length(nliv)==1 then 
   ncont=nliv; 
else 
   ncont=length(nliv);
end
if ~exists("l","local") then l=0; end
if ~exists("amin","local") | ~exists("amax","local") then
   amin=double(min(a));amax=double(max(a));
// It's better to transform amin and amax to double, otherwise
// mixed type vectors or wrong type arguments to x primitives will
// result. Besides, vector-scalar comparisons involving integers are
// still (scilab 2.5, 2.5.1) very buggy.
end
if ~exists("x0","local") | ~exists("y0","local") then
  x0=1:ax; y0=1:ay;  // centers of the rectangles, if none specified
end
if ~exists("aspect","local") then aspect="sq"; end

if aspect=="iso" then strf="040"; else strf="020"; end

if amin>=amax then amin=double(min(a));amax=double(max(a)); end
if amin==amax then amax=amin+1; end

a(double(a)<amin)=amin; a(double(a)>amax)=amax;

x0=matrix(x0,1,-1); y0=matrix(y0,1,-1); 

f=xgetech(); 
nc=xget("lastpattern");
// beware - drivers other than 'REC' have their bugs about colormaps

if ax==0 | ay==0 then return; end

//write(%io(2),string([amin,amax,l,nliv]))

// colorbar:
if modulo(l+3,4)<=1 then
 xsetech([f(1)+0.95*f(3),f(2),0.04*f(3),f(4)]);
 grayplot([0,1],amin+(amax-amin)*(0:nc)/nc,[(0:nc);(0:nc)],'011',...
    [0,amin,1,amax],[0,0,10,10])
// the graduation of the axes of the legend is still ugly;
//  someday I would be better to implement it at low level
 xclea(-0.35,amin-0.01*(amax-amin),1.6,0.15*(amax-amin))
// that's a little of a hack to plot a colorbar, I know
//colorbar contours
 if abs(modulo(l,2))==1 & ax>1 & ay>1 then
   fpfc=xget("fpf"); xset("fpf"," ");      // here no labels for sure
   contour2d([0,1],amin+(amax-amin)*(0:nc)/nc,...
    amin+(amax-amin)*[(0:nc);(0:nc)]/nc,nliv,...
    (nc+1)*ones(1,ncont),'022');
   xset('fpf',fpfc);            // restore status of contour labels
 end
 xsetech([f(1),f(2),0.94*f(3),f(4)]);
end

if ax>1 then
 x=[(3*x0(1)-x0(2))/2, (x0(2:$)+x0(1:$-1))/2, (3*x0($)-x0($-1))/2]
else
 x=[x0-.5,x0+.5]
end
if ay>1 then
 y=[(3*y0(1)-y0(2))/2, (y0(2:$)+y0(1:$-1))/2, (3*y0($)-y0($-1))/2]
else
 y=[y0-.5,y0+.5]
end
dx=x(2:$)-x(1:$-1);
dy=y(2:$)-y(1:$-1);

// a boundary to fix plot coordinates
plot2d([x(1),x($)],[y(1),y($)],0,strf)
//plotframe([x(1),y(1),x($),y($)],[-1 -1 -1 -1]) 
//puts digits -I don't like it

xset("clipgrf");  // this is necessary for pretty zooming 
                 //  it's undocumented in help xset... (2.4.1)

if l/4 <1 then
//pixel map:

//the idea of the loop, for not exceeding stacksize, I copied
// from $SCI/macros/int/%i_Matplot.sci
// It fails in one (pathological) case: ay=1 and ax too large.
  yl=0
  Max=stacksize();Max=int((Max(1)-Max(2))/18)
//intermediate storage can grow up to 12*prod(size(a)), why do I need 18?
  step=max(1,int(Max/ax))
  while yl<ay
    yl=max(yl,1)  // the loop has to be executed once if ay=1
    yl1=min(yl+step-1,ay);                                                           
    aly=yl1-yl+1
    ffill=round(matrix(1+(nc-1)*(double(a(:,yl:yl1))-amin)/(amax-amin),..
           ax*aly,1));
//and this makes 2 times double(a) [could be just a bit less if I'd
//  use int16(ffill)]

//reduction  of the number of rectangles:
// find the only rectangles which are not on the same line 
//  and don't get the same color:
    rowstop=ffill(2:$)==ffill(1:($-1));
    if aly>1 then rowstop(ax:ax:$)=%f; end  
//the if because  bool([])=something gives error
    ired=find([%T, ~rowstop']);
    clear rowstop
//RAM: 2 a + nred
    nred=length(ired); 
// reduce first ffill
    ffill=ffill(ired);
//RAM: a +  2 nred
    mredx=modulo(ired-1,ax)+1;
//RAM: a +  3 nred
//expand the width of the remaining rectangles: to x of the next one
    xred=x(mredx);  qred=ired(1:($-1));
//RAM: a +  5 nred
    rett=matrix(dx'*ones(1,aly),ax*aly,1)';
//this way the point (i,j) falls in the center of the i,jth colored
// rectangle
//RAM: 2 a + 5 nred.
    rett(qred) = xred(2:$)-xred(1:$-1);
//the latter will have a wrong sign if two rectangles are not
// on the same line: then expand the rectangle only to the end of
// that line 
    zred=qred(mredx(2:$)==1);  
//RAM: 2 a +  6 nred
    rett([zred,ired($)]) = x($)-x(modulo([zred,ired($)]-1,ax)+1);
    clear qred zred mredx
//RAM: 2 a +  3 nred
//the last unique rectangle goes for sure to the end of the line
//now reduce rect
    rett=rett(ired);
//RAM: 1 a +  4 nred
//write(%io(2),string(ax*aly)+' rects;'+string(nred)+' uniq')

// Using instead polylines insted of xrects:  
    xpols=[xred; xred+rett; xred+rett; xred]
//RAM: 1 a +  8 nred
    clear rett xred;
//RAM: 1 a +  6 nred
    dredx=yl+(ired-1)/ax; ddy=dy(dredx); yred=y(dredx+1);
//RAM: 1 a +  9 nred
    clear ired dredx
//RAM: 1 a +  7 nred
    ypols=[yred;yred;yred-ddy;yred-ddy];
//RAM: 1 a +  11 nred
//here I filled up to 12 times double(a) (if there was no reduction)
    clear ddy yred
//RAM back to 1 a + 9 nred
    xfpolys(xpols,ypols,-ffill)
//unfortunately xfpolys doesn't accept integers, otherwise some
// economization of space could be possible and desirable
    clear xpols ypols ffill;
// finally, here I should be back to a ram occupation of double(a)
  yl=yl1
  end
else
  fpfc=xget("fpf"); xset("fpf"," ");      // also here no labels
                                          // but is contourf aware?? 
//  xset("clipgrf");     // clipping doesn't seem to work here...
  contourf([x(1),x0,x($)],[y(1),y0,y($)],...
  double([a(1,1),a(1,:),a(1,$);a(:,1),a,a(:,$);...
          a($,1),a($,:),a($,$)]),...
    amin:(amax-amin)/(nc):amax,0:nc,strf);
  xset("fp",fpfc);
  xset("pattern",nc+1)   // ?? it seems this is not reset by contourf
end

 
//contour
if abs(modulo(l,2))==1 & ax>1 & ay>1 then
  contour2d([x(1),x0,x($)],[y(1),y0,y($)],...
            double([a(1,1),a(1,:),a(1,$);a(:,1),a,a(:,$);...
                    a($,1),a($,:),a($,$)]),...
                    nliv,(nc+1)*ones(1,ncont),strf);
// contour2d can contour integers by itself only in 2.5.1;
// anyway the above fails in 2.5, I suppose it's a bug there
    xset("clipgrf") // this is needed again for clipping the frame
                    // when zooming
end
// a frame, just for aesthetics
//plot2d([x(1),x($)],[y(1),y($)],0,strf)  
// ticking is problematic because it is based on x,y and not on x0,y0.
// I just leave a plain box.
xrect(min(x),max(y),abs(x($)-x(1)),abs(y($)-y(1)))
xset("clipoff")
// here I could also add ticks, but I'd like that xaxis would be
//  stable (was changed in 2.5.1 unstable 21/7/2000); besides,
//  I would like to make some options selectable. Another time.

//restore the plotting area if it was split by colorbar+map
if modulo(l+3,4)<=1 then xsetech(f); end

return

